"""
Wrangl implements two learning boilerplates:

- `wrangl.learn.supervised` for Supervised Learning, mostly using PytorchLightning.

.. include:: ../examples/learn/README.md
"""
from .supervised import SupervisedModel
